!******************************************************************************!
!                                                                              !
!    class_colorflow.f90                                                       !
!    is part of RECOLA2 (REcursive Computation of One Loop Amplitudes)2        !
!                                                                              !
!    Copyright (C) 2016-2018 Ansgar Denner, Jean-Nicolas Lang and              !
!                            Sandro Uccirati                                   !
!                                                                              !
!    RECOLA2 is licenced under the GNU GPL version 3,                          !
!    see COPYING for details.                                                  !
!                                                                              !
!******************************************************************************!
!                                                           .---.              !
!                                                          /  .  \             !
!                                                         |\_/|   |            !
!                                                         |   |  /|            !
!          .----------------------------------------------------' |            !
!         /  .-.   class_colorflow.f90                            |            !
!        |  /   \  This code was generated with REPT1L.           |            !
!        | |\_.  | Change this file and you get what you deserve. |            !
!        |\|  | /| Author: J.N. Lang                              |            !
!        | `---' | UFO date: 07/2017                              |            !
!        |       | UFO version: 1.0                               |            !
!        |       | Generation date: 2018-08-14 21:48             /             !
!        |       |----------------------------------------------'              !
!        \       |                                                             !
!         \     /                                                              !
!          `---'                                                               !
!******************************************************************************!
module class_colourflow
  use input_mdl, only: error_mdl
  use class_vertices
  implicit none
  contains
    subroutine compute_colour_correlation_mdl(colorflowIds, csIn, U1GluonIn,   &
    csOut, U1GluonOut, csLast, select_branch, last, pr, lp, legsE, ncpower,    &
    n_In)
      integer, dimension(:),     intent(inout) :: select_branch
      integer, dimension(:,-1:), intent(in)    :: csIn
      integer, allocatable,      intent(inout) :: csOut(:,:)
      logical, dimension(:),     intent(in)    :: U1GluonIn
      logical,                   intent(inout) :: U1GluonOut
      integer, dimension(-1:0),  intent(in)    :: csLast
      logical,                   intent(in)    :: last
      integer,                   intent(in)    :: pr,legsE,lp,n_In
      integer, dimension(0:),    intent(in)    :: colorflowIds
      integer, dimension(:),     intent(inout) :: ncpower
      integer :: nbranches
      nbranches = size(select_branch)
      select case(n_In)
      case(1)
        call compute_colour_correlation2(nbranches, colorflowIds, csIn,        &
        U1GluonIn, csOut, U1GluonOut, csLast, select_branch, last, pr, lp,     &
        legsE, ncpower)
      case(2)
        call compute_colour_correlation3(nbranches, colorflowIds, csIn,        &
        U1GluonIn, csOut, U1GluonOut, csLast, select_branch, last, pr, lp,     &
        legsE, ncpower)
      case(3)
        call compute_colour_correlation4(nbranches, colorflowIds, csIn,        &
        U1GluonIn, csOut, U1GluonOut, csLast, select_branch, last, pr, lp,     &
        legsE, ncpower)
      case(4)
        call compute_colour_correlation5(nbranches, colorflowIds, csIn,        &
        U1GluonIn, csOut, U1GluonOut, csLast, select_branch, last, pr, lp,     &
        legsE, ncpower)
      case(5)
        call compute_colour_correlation6(nbranches, colorflowIds, csIn,        &
        U1GluonIn, csOut, U1GluonOut, csLast, select_branch, last, pr, lp,     &
        legsE, ncpower)
      case(6)
        call compute_colour_correlation7(nbranches, colorflowIds, csIn,        &
        U1GluonIn, csOut, U1GluonOut, csLast, select_branch, last, pr, lp,     &
        legsE, ncpower)
      case(7)
        call compute_colour_correlation8(nbranches, colorflowIds, csIn,        &
        U1GluonIn, csOut, U1GluonOut, csLast, select_branch, last, pr, lp,     &
        legsE, ncpower)
      case default
        call error_mdl("No colourflow for `n_In`< 1 | `n_In`> 7.", where="     &
        compute_colour_correlation_mdl")
      end select 
    end subroutine compute_colour_correlation_mdl

    subroutine compute_colour_correlation2(nbranches, colorflowIds, csIn,      &
    U1GluonIn, csOut, U1GluonOut, csLast, select_branch, last, pr, lp, legsE,  &
    ncpower)
      integer,                       intent(in)    :: nbranches
      integer, dimension(nbranches), intent(inout) :: select_branch
      integer, dimension(:,-1:),     intent(in)    :: csIn
      integer, allocatable,          intent(inout) :: csOut(:,:)
      logical, dimension(:),         intent(in)    :: U1GluonIn
      logical,                       intent(inout) :: U1GluonOut
      integer, dimension(-1:0),      intent(in)    :: csLast
      logical,                       intent(in)    :: last
      integer,                       intent(in)    :: pr, legsE, lp
      integer, dimension(0:),        intent(in)    :: colorflowIds
      integer, dimension(nbranches), intent(inout) :: ncpower

      integer, dimension(-1:legsE,nbranches) :: csTmp
      integer :: bin_color  ! Number of valid colorflows
      integer :: cf
      integer :: i, j, ib
      bin_color = 1
      cf = 0
      do while(cf < size(colorflowIds))
        ! select_branch is zero if it has been filtered before, and the
        ! corresponding colorflow is not computed
        if (select_branch(cf+1) == 0) then
          cf = cf+1
          cycle
        end if
        csTmp(-1:legsE, bin_color) = 0 ! Init as colorless
        do i = 1, legsE
          do j = 1, size(csIn,1)
            ! Copying color deltas
            csTmp(i, bin_color) = csTmp(i, bin_color) + csIn(j,i)
          end do
        end do
        select case(colorflowIds(cf))
!       d^2_1
        case(42)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_1 d^2_2
        case(16)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(1,-1), bin_color) = csIn(1,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^1_2
        case(14)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = 0
          if (last) then
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_1
        case(15)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
          U1GluonOut = U1GluonIn(1)
        case(13)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
        case default
          call error_mdl('Unknown colourflow id.', where                       &
          ='compute_colour_correlation2')
        end select 
        cf = cf +1
        bin_color = bin_color + 1
      end do 
      bin_color = bin_color - 1
      if (last .and. (lp .eq.1)) then
        do ib = 1, bin_color
          if (csTmp(legsE - 1, ib).eq.legsE) then
            ncpower(ib) = ncpower(ib) + 1
          else
            do j = 1, legsE
              if (csTmp(j, ib).eq.legsE) then
                csTmp(j, ib) = csTmp(legsE - 1, ib)
              end if
            end do
          end if
          csTmp(legsE - 1, ib) = 0
          if (csTmp(legsE, ib) .eq. legsE - 1) then
            ncpower(ib) = ncpower(ib) + 1
          else
            do j = 1, legsE
              if (csTmp(j, ib).eq.legsE - 1) then
                csTmp(j, ib) = csTmp(legsE, ib)
              end if
            end do
          end if
          csTmp(legsE, ib) = 0
        end do
      end if
      ! Outgoing colorflow is adjusted to the actual number of valid branches
      allocate(csOut(-1:legsE, bin_color))
      do ib = 1, bin_color
        csOut(-1:legsE, ib) = csTmp(-1:legsE, ib)
      end do
      ! select_branch has been updated and valid branches are filled gapless.
      ! non-valid branches are set to zero.
      if (bin_color .lt. size(select_branch)) then
        select_branch(bin_color+1:size(select_branch)) = 0
      end if
    end subroutine compute_colour_correlation2

    subroutine compute_colour_correlation3(nbranches, colorflowIds, csIn,      &
    U1GluonIn, csOut, U1GluonOut, csLast, select_branch, last, pr, lp, legsE,  &
    ncpower)
      integer,                       intent(in)    :: nbranches
      integer, dimension(nbranches), intent(inout) :: select_branch
      integer, dimension(:,-1:),     intent(in)    :: csIn
      integer, allocatable,          intent(inout) :: csOut(:,:)
      logical, dimension(:),         intent(in)    :: U1GluonIn
      logical,                       intent(inout) :: U1GluonOut
      integer, dimension(-1:0),      intent(in)    :: csLast
      logical,                       intent(in)    :: last
      integer,                       intent(in)    :: pr, legsE, lp
      integer, dimension(0:),        intent(in)    :: colorflowIds
      integer, dimension(nbranches), intent(inout) :: ncpower

      integer, dimension(-1:legsE,nbranches) :: csTmp
      integer :: bin_color  ! Number of valid colorflows
      integer :: cf
      integer :: i, j, ib
      bin_color = 1
      cf = 0
      do while(cf < size(colorflowIds))
        ! select_branch is zero if it has been filtered before, and the
        ! corresponding colorflow is not computed
        if (select_branch(cf+1) == 0) then
          cf = cf+1
          cycle
        end if
        csTmp(-1:legsE, bin_color) = 0 ! Init as colorless
        do i = 1, legsE
          do j = 1, size(csIn,1)
            ! Copying color deltas
            csTmp(i, bin_color) = csTmp(i, bin_color) + csIn(j,i)
          end do
        end do
        select case(colorflowIds(cf))
!       d^1_3 d^3_1
        case(45)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
          U1GluonOut = U1GluonIn(1)
!       d^1_3 d^3_2
        case(11)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_1
        case(19)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_2 d^1_1
        case(31)
          if (U1GluonIn(1)) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = csIn(2,0)
            csTmp(csIn(1,-1), bin_color) = csIn(1,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^3_2 d^2_1
        case(26)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_3 d^1_1
        case(29)
          if (U1GluonIn(1)) then
            csTmp(-1,bin_color) = csIn(2,-1)
            csTmp(0,bin_color) = 0
            csTmp(csIn(1,-1), bin_color) = csIn(1,0)
            if (last) then
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_1 d^2_3 d^3_2
        case(51)
          if (U1GluonIn(1)) then
            csTmp(-1,bin_color) = csIn(2,-1)
            csTmp(0,bin_color) = csIn(2,0)
            csTmp(csIn(1,-1), bin_color) = csIn(1,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^3_1 d^1_2
        case(30)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_1 d^2_2
        case(27)
          if (U1GluonIn(2)) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = csIn(1,0)
            csTmp(csIn(2,-1), bin_color) = csIn(2,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^2_1 d^1_3
        case(28)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_2 d^1_3 d^2_1
        case(35)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_2
        case(21)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_3
        case(22)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^2_1 d^3_2
        case(1)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_3 d^3_1
        case(24)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_3 d^1_2 d^3_1
        case(33)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_2 d^3_3
        case(44)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(2,-1), bin_color) = csIn(2,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^1_2 d^3_3
        case(12)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(1,-1), bin_color) = csIn(2,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^3_1 d^1_2 d^2_3
        case(34)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_3 d^3_2
        case(43)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
          U1GluonOut = U1GluonIn(2)
!       d^2_1 d^3_3
        case(25)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(2,-1), bin_color) = csIn(1,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^2_3
        case(20)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = 0
          if (last) then
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3
        case(17)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = 0
          if (last) then
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_1
        case(47)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_2 d^2_1 d^1_3
        case(38)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_1
        case(18)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
        case(8)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_3 d^3_1
        case(0)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_1 d^1_3 d^3_2
        case(32)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^2_2 d^3_1
        case(49)
          if (U1GluonIn(2)) then
            csTmp(-1,bin_color) = csIn(1,-1)
            csTmp(0,bin_color) = csIn(1,0)
            csTmp(csIn(2,-1), bin_color) = csIn(2,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_1 d^3_3
        case(46)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(1,-1), bin_color) = csIn(1,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^3_1 d^2_3 d^1_2
        case(39)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_1 d^3_2 d^1_3
        case(41)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_1 d^3_3
        case(50)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(1,-1), bin_color) = csIn(2,0)
            csTmp(csIn(2,-1), bin_color) = csIn(1,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^1_1 d^2_2
        case(48)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(1,0)
          csTmp(csIn(2,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^3_1 d^2_3
        case(37)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^2_2
        case(23)
          if (U1GluonIn(2)) then
            csTmp(-1,bin_color) = csIn(1,-1)
            csTmp(0,bin_color) = 0
            csTmp(csIn(2,-1), bin_color) = csIn(2,0)
            if (last) then
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_1 d^2_2 d^3_3
        case(52)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(1,-1), bin_color) = csIn(1,0)
            csTmp(csIn(2,-1), bin_color) = csIn(2,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^1_2
        case(10)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_3 d^3_1 d^1_2
        case(40)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^3_2 d^2_1
        case(36)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
        case default
          call error_mdl('Unknown colourflow id.', where                       &
          ='compute_colour_correlation3')
        end select 
        cf = cf +1
        bin_color = bin_color + 1
      end do 
      bin_color = bin_color - 1
      if (last .and. (lp .eq.1)) then
        do ib = 1, bin_color
          if (csTmp(legsE - 1, ib).eq.legsE) then
            ncpower(ib) = ncpower(ib) + 1
          else
            do j = 1, legsE
              if (csTmp(j, ib).eq.legsE) then
                csTmp(j, ib) = csTmp(legsE - 1, ib)
              end if
            end do
          end if
          csTmp(legsE - 1, ib) = 0
          if (csTmp(legsE, ib) .eq. legsE - 1) then
            ncpower(ib) = ncpower(ib) + 1
          else
            do j = 1, legsE
              if (csTmp(j, ib).eq.legsE - 1) then
                csTmp(j, ib) = csTmp(legsE, ib)
              end if
            end do
          end if
          csTmp(legsE, ib) = 0
        end do
      end if
      ! Outgoing colorflow is adjusted to the actual number of valid branches
      allocate(csOut(-1:legsE, bin_color))
      do ib = 1, bin_color
        csOut(-1:legsE, ib) = csTmp(-1:legsE, ib)
      end do
      ! select_branch has been updated and valid branches are filled gapless.
      ! non-valid branches are set to zero.
      if (bin_color .lt. size(select_branch)) then
        select_branch(bin_color+1:size(select_branch)) = 0
      end if
    end subroutine compute_colour_correlation3

    subroutine compute_colour_correlation4(nbranches, colorflowIds, csIn,      &
    U1GluonIn, csOut, U1GluonOut, csLast, select_branch, last, pr, lp, legsE,  &
    ncpower)
      integer,                       intent(in)    :: nbranches
      integer, dimension(nbranches), intent(inout) :: select_branch
      integer, dimension(:,-1:),     intent(in)    :: csIn
      integer, allocatable,          intent(inout) :: csOut(:,:)
      logical, dimension(:),         intent(in)    :: U1GluonIn
      logical,                       intent(inout) :: U1GluonOut
      integer, dimension(-1:0),      intent(in)    :: csLast
      logical,                       intent(in)    :: last
      integer,                       intent(in)    :: pr, legsE, lp
      integer, dimension(0:),        intent(in)    :: colorflowIds
      integer, dimension(nbranches), intent(inout) :: ncpower

      integer, dimension(-1:legsE,nbranches) :: csTmp
      integer :: bin_color  ! Number of valid colorflows
      integer :: cf
      integer :: i, j, ib
      bin_color = 1
      cf = 0
      do while(cf < size(colorflowIds))
        ! select_branch is zero if it has been filtered before, and the
        ! corresponding colorflow is not computed
        if (select_branch(cf+1) == 0) then
          cf = cf+1
          cycle
        end if
        csTmp(-1:legsE, bin_color) = 0 ! Init as colorless
        do i = 1, legsE
          do j = 1, size(csIn,1)
            ! Copying color deltas
            csTmp(i, bin_color) = csTmp(i, bin_color) + csIn(j,i)
          end do
        end do
        select case(colorflowIds(cf))
!       d^2_2 d^4_4
        case(58)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(2,-1), bin_color) = csIn(2,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^1_2 d^2_3 d^3_1 d^4_4
        case(92)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(1,-1), bin_color) = csIn(2,0)
            csTmp(csIn(2,-1), bin_color) = csIn(3,0)
            csTmp(csIn(3,-1), bin_color) = csIn(1,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^1_3 d^2_2 d^3_1
        case(73)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(2,-1), bin_color) = csIn(2,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^2_1 d^4_2
        case(77)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^2_4 d^3_2 d^4_1
        case(2)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^2_4 d^3_1 d^4_2
        case(94)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_1 d^2_4 d^3_2 d^4_3
        case(95)
          if (U1GluonIn(1)) then
            csTmp(-1,bin_color) = csIn(2,-1)
            csTmp(0,bin_color) = csIn(3,0)
            csTmp(csIn(1,-1), bin_color) = csIn(1,0)
            csTmp(csIn(3,-1), bin_color) = csIn(2,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_3 d^2_1 d^3_2 d^4_4
        case(97)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(1,-1), bin_color) = csIn(3,0)
            csTmp(csIn(2,-1), bin_color) = csIn(1,0)
            csTmp(csIn(3,-1), bin_color) = csIn(2,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^1_4 d^2_3 d^3_2 d^4_1
        case(91)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_1 d^3_3 d^4_4
        case(101)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(1,-1), bin_color) = csIn(2,0)
            csTmp(csIn(2,-1), bin_color) = csIn(1,0)
            csTmp(csIn(3,-1), bin_color) = csIn(3,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^1_1 d^2_2 d^3_3
        case(75)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(1,0)
          csTmp(csIn(2,-1), bin_color) = csIn(2,0)
          csTmp(csIn(3,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_1 d^2_3 d^3_2
        case(72)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(1,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^4_1
        case(61)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
          U1GluonOut = U1GluonIn(1)
!       d^3_4 d^4_3
        case(53)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
          U1GluonOut = U1GluonIn(3)
!       d^1_2 d^2_1
        case(63)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_3 d^3_4 d^4_2
        case(65)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_3 d^4_4
        case(54)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(3,-1), bin_color) = csIn(3,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^2_2 d^3_3
        case(56)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(2,0)
          csTmp(csIn(3,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^2_1 d^3_2 d^4_3
        case(5)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_2 d^3_3 d^4_4
        case(68)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(2,-1), bin_color) = csIn(2,0)
            csTmp(csIn(3,-1), bin_color) = csIn(3,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^2_4 d^3_3 d^4_2
        case(66)
          if (U1GluonIn(3)) then
            csTmp(-1,bin_color) = csIn(2,-1)
            csTmp(0,bin_color) = csIn(2,0)
            csTmp(csIn(3,-1), bin_color) = csIn(3,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_1 d^3_3 d^4_4
        case(87)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(1,-1), bin_color) = csIn(1,0)
            csTmp(csIn(3,-1), bin_color) = csIn(3,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^1_3 d^2_1 d^3_2
        case(71)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_1 d^4_4
        case(80)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(1,-1), bin_color) = csIn(2,0)
            csTmp(csIn(2,-1), bin_color) = csIn(1,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^1_4 d^2_2 d^4_1
        case(79)
          if (U1GluonIn(2)) then
            csTmp(-1,bin_color) = csIn(1,-1)
            csTmp(0,bin_color) = csIn(1,0)
            csTmp(csIn(2,-1), bin_color) = csIn(2,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_1 d^2_2 d^4_4
        case(81)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(1,-1), bin_color) = csIn(1,0)
            csTmp(csIn(2,-1), bin_color) = csIn(2,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^1_1 d^3_3
        case(60)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(1,0)
          csTmp(csIn(3,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_1 d^2_3 d^3_4 d^4_2
        case(96)
          if (U1GluonIn(1)) then
            csTmp(-1,bin_color) = csIn(3,-1)
            csTmp(0,bin_color) = csIn(2,0)
            csTmp(csIn(1,-1), bin_color) = csIn(1,0)
            csTmp(csIn(2,-1), bin_color) = csIn(3,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^2_3 d^3_2 d^4_4
        case(69)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(2,-1), bin_color) = csIn(3,0)
            csTmp(csIn(3,-1), bin_color) = csIn(2,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^1_3 d^3_4 d^4_1
        case(88)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
        case(9)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_1 d^2_3 d^3_2 d^4_4
        case(106)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(1,-1), bin_color) = csIn(1,0)
            csTmp(csIn(2,-1), bin_color) = csIn(3,0)
            csTmp(csIn(3,-1), bin_color) = csIn(2,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^1_1 d^2_2 d^3_4 d^4_3
        case(105)
          if (U1GluonIn(2)) then
            csTmp(-1,bin_color) = csIn(3,-1)
            csTmp(0,bin_color) = csIn(3,0)
            csTmp(csIn(1,-1), bin_color) = csIn(1,0)
            csTmp(csIn(2,-1), bin_color) = csIn(2,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_4 d^2_3 d^3_1 d^4_2
        case(3)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_1 d^2_4 d^4_2
        case(78)
          if (U1GluonIn(1)) then
            csTmp(-1,bin_color) = csIn(2,-1)
            csTmp(0,bin_color) = csIn(2,0)
            csTmp(csIn(1,-1), bin_color) = csIn(1,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_3 d^3_1
        case(59)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_1 d^4_4
        case(62)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(1,-1), bin_color) = csIn(1,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^1_4 d^3_1 d^4_3
        case(83)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^2_2 d^3_1 d^4_3
        case(90)
          if (U1GluonIn(2)) then
            csTmp(-1,bin_color) = csIn(1,-1)
            csTmp(0,bin_color) = csIn(3,0)
            csTmp(csIn(2,-1), bin_color) = csIn(2,0)
            csTmp(csIn(3,-1), bin_color) = csIn(1,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^2_4 d^3_2 d^4_3
        case(70)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_2 d^3_4 d^4_3
        case(67)
          if (U1GluonIn(2)) then
            csTmp(-1,bin_color) = csIn(3,-1)
            csTmp(0,bin_color) = csIn(3,0)
            csTmp(csIn(2,-1), bin_color) = csIn(2,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_4 d^2_2 d^3_3 d^4_1
        case(100)
          if (U1GluonIn(3)) then
            csTmp(-1,bin_color) = csIn(1,-1)
            csTmp(0,bin_color) = csIn(1,0)
            csTmp(csIn(2,-1), bin_color) = csIn(2,0)
            csTmp(csIn(3,-1), bin_color) = csIn(3,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_2 d^2_3 d^3_1
        case(76)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_1 d^2_2 d^3_3 d^4_4
        case(103)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(1,-1), bin_color) = csIn(1,0)
            csTmp(csIn(2,-1), bin_color) = csIn(2,0)
            csTmp(csIn(3,-1), bin_color) = csIn(3,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^1_2 d^2_4 d^3_1 d^4_3
        case(4)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_1 d^3_4 d^4_3
        case(84)
          if (U1GluonIn(1)) then
            csTmp(-1,bin_color) = csIn(3,-1)
            csTmp(0,bin_color) = csIn(3,0)
            csTmp(csIn(1,-1), bin_color) = csIn(1,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_1 d^2_2
        case(64)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(1,0)
          csTmp(csIn(2,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^2_2 d^3_4 d^4_1
        case(98)
          if (U1GluonIn(2)) then
            csTmp(-1,bin_color) = csIn(3,-1)
            csTmp(0,bin_color) = csIn(1,0)
            csTmp(csIn(1,-1), bin_color) = csIn(3,0)
            csTmp(csIn(2,-1), bin_color) = csIn(2,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_2 d^2_4 d^3_3 d^4_1
        case(99)
          if (U1GluonIn(3)) then
            csTmp(-1,bin_color) = csIn(2,-1)
            csTmp(0,bin_color) = csIn(1,0)
            csTmp(csIn(1,-1), bin_color) = csIn(2,0)
            csTmp(csIn(3,-1), bin_color) = csIn(3,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_1 d^2_4 d^3_3 d^4_2
        case(104)
          if (U1GluonIn(3)) then
            csTmp(-1,bin_color) = csIn(2,-1)
            csTmp(0,bin_color) = csIn(2,0)
            csTmp(csIn(1,-1), bin_color) = csIn(1,0)
            csTmp(csIn(3,-1), bin_color) = csIn(3,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^2_3 d^3_2
        case(55)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^2_1 d^3_3 d^4_2
        case(89)
          if (U1GluonIn(3)) then
            csTmp(-1,bin_color) = csIn(1,-1)
            csTmp(0,bin_color) = csIn(2,0)
            csTmp(csIn(2,-1), bin_color) = csIn(1,0)
            csTmp(csIn(3,-1), bin_color) = csIn(3,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_3 d^2_2 d^3_1 d^4_4
        case(102)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(1,-1), bin_color) = csIn(3,0)
            csTmp(csIn(2,-1), bin_color) = csIn(2,0)
            csTmp(csIn(3,-1), bin_color) = csIn(1,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^1_2 d^2_1 d^3_4 d^4_3
        case(93)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_3 d^3_4 d^4_1
        case(7)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_1 d^3_3
        case(74)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          csTmp(csIn(3,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_4 d^4_2
        case(57)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
          U1GluonOut = U1GluonIn(2)
!       d^1_2 d^2_4 d^4_1
        case(82)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^3_1 d^4_4
        case(86)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(1,-1), bin_color) = csIn(3,0)
            csTmp(csIn(3,-1), bin_color) = csIn(1,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^1_3 d^2_1 d^3_4 d^4_2
        case(6)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^3_3 d^4_1
        case(85)
          if (U1GluonIn(3)) then
            csTmp(-1,bin_color) = csIn(1,-1)
            csTmp(0,bin_color) = csIn(1,0)
            csTmp(csIn(3,-1), bin_color) = csIn(3,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
        case default
          call error_mdl('Unknown colourflow id.', where                       &
          ='compute_colour_correlation4')
        end select 
        cf = cf +1
        bin_color = bin_color + 1
      end do 
      bin_color = bin_color - 1
      if (last .and. (lp .eq.1)) then
        do ib = 1, bin_color
          if (csTmp(legsE - 1, ib).eq.legsE) then
            ncpower(ib) = ncpower(ib) + 1
          else
            do j = 1, legsE
              if (csTmp(j, ib).eq.legsE) then
                csTmp(j, ib) = csTmp(legsE - 1, ib)
              end if
            end do
          end if
          csTmp(legsE - 1, ib) = 0
          if (csTmp(legsE, ib) .eq. legsE - 1) then
            ncpower(ib) = ncpower(ib) + 1
          else
            do j = 1, legsE
              if (csTmp(j, ib).eq.legsE - 1) then
                csTmp(j, ib) = csTmp(legsE, ib)
              end if
            end do
          end if
          csTmp(legsE, ib) = 0
        end do
      end if
      ! Outgoing colorflow is adjusted to the actual number of valid branches
      allocate(csOut(-1:legsE, bin_color))
      do ib = 1, bin_color
        csOut(-1:legsE, ib) = csTmp(-1:legsE, ib)
      end do
      ! select_branch has been updated and valid branches are filled gapless.
      ! non-valid branches are set to zero.
      if (bin_color .lt. size(select_branch)) then
        select_branch(bin_color+1:size(select_branch)) = 0
      end if
    end subroutine compute_colour_correlation4

    subroutine compute_colour_correlation5(nbranches, colorflowIds, csIn,      &
    U1GluonIn, csOut, U1GluonOut, csLast, select_branch, last, pr, lp, legsE,  &
    ncpower)
      integer,                       intent(in)    :: nbranches
      integer, dimension(nbranches), intent(inout) :: select_branch
      integer, dimension(:,-1:),     intent(in)    :: csIn
      integer, allocatable,          intent(inout) :: csOut(:,:)
      logical, dimension(:),         intent(in)    :: U1GluonIn
      logical,                       intent(inout) :: U1GluonOut
      integer, dimension(-1:0),      intent(in)    :: csLast
      logical,                       intent(in)    :: last
      integer,                       intent(in)    :: pr, legsE, lp
      integer, dimension(0:),        intent(in)    :: colorflowIds
      integer, dimension(nbranches), intent(inout) :: ncpower

      integer, dimension(-1:legsE,nbranches) :: csTmp
      integer :: bin_color  ! Number of valid colorflows
      integer :: cf
      integer :: i, j, ib
      bin_color = 1
      cf = 0
      do while(cf < size(colorflowIds))
        ! select_branch is zero if it has been filtered before, and the
        ! corresponding colorflow is not computed
        if (select_branch(cf+1) == 0) then
          cf = cf+1
          cycle
        end if
        csTmp(-1:legsE, bin_color) = 0 ! Init as colorless
        do i = 1, legsE
          do j = 1, size(csIn,1)
            ! Copying color deltas
            csTmp(i, bin_color) = csTmp(i, bin_color) + csIn(j,i)
          end do
        end do
        select case(colorflowIds(cf))
        case default
          call error_mdl('Unknown colourflow id.', where                       &
          ='compute_colour_correlation5')
        end select 
        cf = cf +1
        bin_color = bin_color + 1
      end do 
      bin_color = bin_color - 1
      if (last .and. (lp .eq.1)) then
        do ib = 1, bin_color
          if (csTmp(legsE - 1, ib).eq.legsE) then
            ncpower(ib) = ncpower(ib) + 1
          else
            do j = 1, legsE
              if (csTmp(j, ib).eq.legsE) then
                csTmp(j, ib) = csTmp(legsE - 1, ib)
              end if
            end do
          end if
          csTmp(legsE - 1, ib) = 0
          if (csTmp(legsE, ib) .eq. legsE - 1) then
            ncpower(ib) = ncpower(ib) + 1
          else
            do j = 1, legsE
              if (csTmp(j, ib).eq.legsE - 1) then
                csTmp(j, ib) = csTmp(legsE, ib)
              end if
            end do
          end if
          csTmp(legsE, ib) = 0
        end do
      end if
      ! Outgoing colorflow is adjusted to the actual number of valid branches
      allocate(csOut(-1:legsE, bin_color))
      do ib = 1, bin_color
        csOut(-1:legsE, ib) = csTmp(-1:legsE, ib)
      end do
      ! select_branch has been updated and valid branches are filled gapless.
      ! non-valid branches are set to zero.
      if (bin_color .lt. size(select_branch)) then
        select_branch(bin_color+1:size(select_branch)) = 0
      end if
    end subroutine compute_colour_correlation5

    subroutine compute_colour_correlation6(nbranches, colorflowIds, csIn,      &
    U1GluonIn, csOut, U1GluonOut, csLast, select_branch, last, pr, lp, legsE,  &
    ncpower)
      integer,                       intent(in)    :: nbranches
      integer, dimension(nbranches), intent(inout) :: select_branch
      integer, dimension(:,-1:),     intent(in)    :: csIn
      integer, allocatable,          intent(inout) :: csOut(:,:)
      logical, dimension(:),         intent(in)    :: U1GluonIn
      logical,                       intent(inout) :: U1GluonOut
      integer, dimension(-1:0),      intent(in)    :: csLast
      logical,                       intent(in)    :: last
      integer,                       intent(in)    :: pr, legsE, lp
      integer, dimension(0:),        intent(in)    :: colorflowIds
      integer, dimension(nbranches), intent(inout) :: ncpower

      integer, dimension(-1:legsE,nbranches) :: csTmp
      integer :: bin_color  ! Number of valid colorflows
      integer :: cf
      integer :: i, j, ib
      bin_color = 1
      cf = 0
      do while(cf < size(colorflowIds))
        ! select_branch is zero if it has been filtered before, and the
        ! corresponding colorflow is not computed
        if (select_branch(cf+1) == 0) then
          cf = cf+1
          cycle
        end if
        csTmp(-1:legsE, bin_color) = 0 ! Init as colorless
        do i = 1, legsE
          do j = 1, size(csIn,1)
            ! Copying color deltas
            csTmp(i, bin_color) = csTmp(i, bin_color) + csIn(j,i)
          end do
        end do
        select case(colorflowIds(cf))
        case default
          call error_mdl('Unknown colourflow id.', where                       &
          ='compute_colour_correlation6')
        end select 
        cf = cf +1
        bin_color = bin_color + 1
      end do 
      bin_color = bin_color - 1
      if (last .and. (lp .eq.1)) then
        do ib = 1, bin_color
          if (csTmp(legsE - 1, ib).eq.legsE) then
            ncpower(ib) = ncpower(ib) + 1
          else
            do j = 1, legsE
              if (csTmp(j, ib).eq.legsE) then
                csTmp(j, ib) = csTmp(legsE - 1, ib)
              end if
            end do
          end if
          csTmp(legsE - 1, ib) = 0
          if (csTmp(legsE, ib) .eq. legsE - 1) then
            ncpower(ib) = ncpower(ib) + 1
          else
            do j = 1, legsE
              if (csTmp(j, ib).eq.legsE - 1) then
                csTmp(j, ib) = csTmp(legsE, ib)
              end if
            end do
          end if
          csTmp(legsE, ib) = 0
        end do
      end if
      ! Outgoing colorflow is adjusted to the actual number of valid branches
      allocate(csOut(-1:legsE, bin_color))
      do ib = 1, bin_color
        csOut(-1:legsE, ib) = csTmp(-1:legsE, ib)
      end do
      ! select_branch has been updated and valid branches are filled gapless.
      ! non-valid branches are set to zero.
      if (bin_color .lt. size(select_branch)) then
        select_branch(bin_color+1:size(select_branch)) = 0
      end if
    end subroutine compute_colour_correlation6

    subroutine compute_colour_correlation7(nbranches, colorflowIds, csIn,      &
    U1GluonIn, csOut, U1GluonOut, csLast, select_branch, last, pr, lp, legsE,  &
    ncpower)
      integer,                       intent(in)    :: nbranches
      integer, dimension(nbranches), intent(inout) :: select_branch
      integer, dimension(:,-1:),     intent(in)    :: csIn
      integer, allocatable,          intent(inout) :: csOut(:,:)
      logical, dimension(:),         intent(in)    :: U1GluonIn
      logical,                       intent(inout) :: U1GluonOut
      integer, dimension(-1:0),      intent(in)    :: csLast
      logical,                       intent(in)    :: last
      integer,                       intent(in)    :: pr, legsE, lp
      integer, dimension(0:),        intent(in)    :: colorflowIds
      integer, dimension(nbranches), intent(inout) :: ncpower

      integer, dimension(-1:legsE,nbranches) :: csTmp
      integer :: bin_color  ! Number of valid colorflows
      integer :: cf
      integer :: i, j, ib
      bin_color = 1
      cf = 0
      do while(cf < size(colorflowIds))
        ! select_branch is zero if it has been filtered before, and the
        ! corresponding colorflow is not computed
        if (select_branch(cf+1) == 0) then
          cf = cf+1
          cycle
        end if
        csTmp(-1:legsE, bin_color) = 0 ! Init as colorless
        do i = 1, legsE
          do j = 1, size(csIn,1)
            ! Copying color deltas
            csTmp(i, bin_color) = csTmp(i, bin_color) + csIn(j,i)
          end do
        end do
        select case(colorflowIds(cf))
        case default
          call error_mdl('Unknown colourflow id.', where                       &
          ='compute_colour_correlation7')
        end select 
        cf = cf +1
        bin_color = bin_color + 1
      end do 
      bin_color = bin_color - 1
      if (last .and. (lp .eq.1)) then
        do ib = 1, bin_color
          if (csTmp(legsE - 1, ib).eq.legsE) then
            ncpower(ib) = ncpower(ib) + 1
          else
            do j = 1, legsE
              if (csTmp(j, ib).eq.legsE) then
                csTmp(j, ib) = csTmp(legsE - 1, ib)
              end if
            end do
          end if
          csTmp(legsE - 1, ib) = 0
          if (csTmp(legsE, ib) .eq. legsE - 1) then
            ncpower(ib) = ncpower(ib) + 1
          else
            do j = 1, legsE
              if (csTmp(j, ib).eq.legsE - 1) then
                csTmp(j, ib) = csTmp(legsE, ib)
              end if
            end do
          end if
          csTmp(legsE, ib) = 0
        end do
      end if
      ! Outgoing colorflow is adjusted to the actual number of valid branches
      allocate(csOut(-1:legsE, bin_color))
      do ib = 1, bin_color
        csOut(-1:legsE, ib) = csTmp(-1:legsE, ib)
      end do
      ! select_branch has been updated and valid branches are filled gapless.
      ! non-valid branches are set to zero.
      if (bin_color .lt. size(select_branch)) then
        select_branch(bin_color+1:size(select_branch)) = 0
      end if
    end subroutine compute_colour_correlation7

    subroutine compute_colour_correlation8(nbranches, colorflowIds, csIn,      &
    U1GluonIn, csOut, U1GluonOut, csLast, select_branch, last, pr, lp, legsE,  &
    ncpower)
      integer,                       intent(in)    :: nbranches
      integer, dimension(nbranches), intent(inout) :: select_branch
      integer, dimension(:,-1:),     intent(in)    :: csIn
      integer, allocatable,          intent(inout) :: csOut(:,:)
      logical, dimension(:),         intent(in)    :: U1GluonIn
      logical,                       intent(inout) :: U1GluonOut
      integer, dimension(-1:0),      intent(in)    :: csLast
      logical,                       intent(in)    :: last
      integer,                       intent(in)    :: pr, legsE, lp
      integer, dimension(0:),        intent(in)    :: colorflowIds
      integer, dimension(nbranches), intent(inout) :: ncpower

      integer, dimension(-1:legsE,nbranches) :: csTmp
      integer :: bin_color  ! Number of valid colorflows
      integer :: cf
      integer :: i, j, ib
      bin_color = 1
      cf = 0
      do while(cf < size(colorflowIds))
        ! select_branch is zero if it has been filtered before, and the
        ! corresponding colorflow is not computed
        if (select_branch(cf+1) == 0) then
          cf = cf+1
          cycle
        end if
        csTmp(-1:legsE, bin_color) = 0 ! Init as colorless
        do i = 1, legsE
          do j = 1, size(csIn,1)
            ! Copying color deltas
            csTmp(i, bin_color) = csTmp(i, bin_color) + csIn(j,i)
          end do
        end do
        select case(colorflowIds(cf))
        case default
          call error_mdl('Unknown colourflow id.', where                       &
          ='compute_colour_correlation8')
        end select 
        cf = cf +1
        bin_color = bin_color + 1
      end do 
      bin_color = bin_color - 1
      if (last .and. (lp .eq.1)) then
        do ib = 1, bin_color
          if (csTmp(legsE - 1, ib).eq.legsE) then
            ncpower(ib) = ncpower(ib) + 1
          else
            do j = 1, legsE
              if (csTmp(j, ib).eq.legsE) then
                csTmp(j, ib) = csTmp(legsE - 1, ib)
              end if
            end do
          end if
          csTmp(legsE - 1, ib) = 0
          if (csTmp(legsE, ib) .eq. legsE - 1) then
            ncpower(ib) = ncpower(ib) + 1
          else
            do j = 1, legsE
              if (csTmp(j, ib).eq.legsE - 1) then
                csTmp(j, ib) = csTmp(legsE, ib)
              end if
            end do
          end if
          csTmp(legsE, ib) = 0
        end do
      end if
      ! Outgoing colorflow is adjusted to the actual number of valid branches
      allocate(csOut(-1:legsE, bin_color))
      do ib = 1, bin_color
        csOut(-1:legsE, ib) = csTmp(-1:legsE, ib)
      end do
      ! select_branch has been updated and valid branches are filled gapless.
      ! non-valid branches are set to zero.
      if (bin_color .lt. size(select_branch)) then
        select_branch(bin_color+1:size(select_branch)) = 0
      end if
    end subroutine compute_colour_correlation8

end module class_colourflow
